#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''

License:
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
The GNU Public License is available at
http://www.gnu.org/copyleft/gpl.html
'''
from gimpfu import *
import sys, os, gettext
import gtk
import subprocess as sub


# https://forums.getpaint.net/topic/23650-gimp-native-format-filetype-plugin-xcf-xcfgz-xcfbz2/
### global variables ###########################################################

whoiam = os.path.abspath(sys.argv[0])

# if not launch within GIMP
def mssgBox(mess):
    flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
    msgBox = gtk.MessageDialog(None, flag, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, mess)
    msgBox.run()
    msgBox.destroy()

try:
    from gimpfu import *
	
except ImportError:
    print("Note: GIMP is needed, '%s' is a plug-in for it.\n"%whoiam)
    # launch from GUI? use mssgBox(text)
    mssgBox("Note: GIMP is needed, '%s' is a plug-in for it.\n"%whoiam)
    sys.exit(1)

# for locale characters in windows file system (by rob_brz1)
def sys_file(f_name):
    sys_encoded = sys.getfilesystemencoding()
    if os.name == 'nt': coded = f_name.encode(sys_encoded)
    else: coded = f_name
    return(coded)

# initialize internationalisation in a user-locale (by rob_brz1)
locale_directory = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "Gimp_to_PaintDotNet", locale_directory, unicode=True )

### main function ##############################################################

def open_file_in_PaintDotNet(image, layer, ink, outputFolder, format, schel, resize, newimage, rem):

    if os.access(sys_file(outputFolder), os.W_OK):
        gimp.progress_init(_("Saving to '") + outputFolder + os.sep + layer.name)
    else:
        gimp.message(_("ERROR: folder '%s' isn't writable, plug-in has terminated.")%outputFolder)
        return
    if ink.find('PaintDotNet') == -1 : return
	
    if format=="xcf":

		xchange_utf = outputFolder + os.sep + layer.name + ".xcf"
		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)
			code=str(pdb.gimp_layer_set_mode(layer, 0))
			exec(code)
					
		pdb.gimp_xcf_save(1, image, layer, xchange_utf, xchange_utf, run_mode=RUN_INTERACTIVE)
	
    if format=="png":
		xchange_utf = outputFolder + os.sep + layer.name + ".png"
		gimp.pdb.file_png_save(image, layer, xchange_utf, xchange_utf, 1, 1, 1, 0, 0, 1, 0, run_mode=RUN_INTERACTIVE)
    if format=="jpg":
		xchange_utf = outputFolder + os.sep + layer.name + ".jpg"
		gimp.pdb.file_jpeg_save(image, layer, xchange_utf, xchange_utf, 1, 0, 0, 0, 1, 0, 1, 0, 0, run_mode=RUN_INTERACTIVE)		
    if format=="tiff":
		xchange_utf = outputFolder + os.sep + layer.name + ".tiff"
		gimp.pdb.file_tiff_save(image, layer, xchange_utf, xchange_utf, 0, run_mode=RUN_INTERACTIVE)
    if format=="bmp":
		xchange_utf = outputFolder + os.sep + layer.name + ".bmp"
		gimp.pdb.file_bmp_save(image, layer, xchange_utf, xchange_utf, run_mode=RUN_INTERACTIVE)
    if format=="webp":
		xchange_utf = outputFolder + os.sep + layer.name + ".webp"	
		pdb.file_webp_save(image, layer, xchange_utf, xchange_utf, 0, 0, 10, 100, 0, 0, 1, 1, 0, 0, 0, 100, 0, run_mode=RUN_INTERACTIVE)	
	
    xchange_sys = sys_file(xchange_utf) # for Python, Win file system not 'utf-8' (by rob_brz1)
    p = sub.Popen([ink, xchange_sys], stdin=sub.PIPE, stdout=sub.PIPE, stderr=sub.STDOUT)
    child_output, child_error = p.communicate()

    if schel:
		newlayer = pdb.gimp_file_load_layer(image, xchange_utf)
		image.add_layer(newlayer,-1)
    if format=="png":		
		gimp.pdb.file_png_load(xchange_utf, layer.name, run_mode=RUN_NONINTERACTIVE)
    if format=="jpg":
		gimp.pdb.file_jpeg_load(xchange_utf, layer.name, run_mode=RUN_NONINTERACTIVE)
    if format=="tiff":		
		gimp.pdb.file_tiff_load(xchange_utf, layer.name, run_mode=RUN_NONINTERACTIVE)
    if format=="bmp":		
		gimp.pdb.file_bmp_load(xchange_utf, layer.name, run_mode=RUN_NONINTERACTIVE)
    if format=="webp":		
		gimp.pdb.file_webp_load(xchange_utf, layer.name, run_mode=RUN_NONINTERACTIVE)
		
	# fit canvas to layers	
    if resize:		
		pdb.gimp_image_resize_to_layers(image)
		
	# open as new image
    if newimage:			
		new_img=pdb.gimp_file_load(xchange_utf, xchange_utf)
		pdb.gimp_display_new(new_img)
	
    # cleanup temp file?
    if rem:
        os.remove(xchange_sys)
    pdb.gimp_displays_flush()
    gimp.progress_init(_("Plug-in end!"))		

### GIMP registring & UI #######################################################

register(
    "open-in-PaintDotNet",
    _("\nThis plugin opens visable layers or all layers for XCF in PaintDotNet. Image name = name active layer.\n")\
    +_("-------------------------Instructions:--------------------------\n")\
    +_("In PaintDotNet: edit image and save changes (overwrite file) and close. This return image to GIMP and end the plug-in.\n")\
    +_("----------------------------------------------------------------\n")\
    +_("Plug-in file-name:\n")+whoiam,
    "Open layer in PaintDotNet, edit image and return to GIMP - V 0.1",
    "MareroQ",
    "GPL",
    "2019",
    _(""u'\u25cf'" Export to PaintDotNet..."),
    "RGB*,GRAY*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "filename", "Start exchange file", None),
        (PF_FILENAME, "ink", _("PaintDotNet executable file: "), "C:\\Program Files\\paint.net\\PaintDotNet.exe"),
        (PF_DIRNAME, "outputFolder", _("Output directory to save temp file:"), "c:\\Users\Joanna\\AppData\\Roaming\\GIMP\\2.10\\tmp"),
        (PF_RADIO, "format", ("Format (temp file)\nto save (interactive) as :"), "png", (("XCF (will convert all layers to \nmode: NORMAL-LEGACY)", "xcf"),("PNG", "png"),("JPG", "jpg"),("TIFF (no save layers)", "tiff"),("BMP", "bmp"),("WEBP (no animation)", "webp"))),
        (PF_BOOL, "schel", _("Return image (as new layer)\nfrom PaintDotNet to GIMP? "), True),
        (PF_BOOL, "resize_1", _("Fit canvas to layers? "), True),
        (PF_BOOL, "newimage_1", _("Open as a new image? "), True),		
        (PF_BOOL, "rem_1", _("Delete temp file? "), False),
    ],
    [],
    open_file_in_PaintDotNet,
    menu="<Image>/File/Export/",
    domain=("Gimp_to_PaintDotNet", locale_directory)
    )

main()
